#include "Headers/patch.h"
#include "Headers/mainwindow.h"
#include "Headers/game.h"

ShopCodeOffsets_s off_Retail = {0xA0,0x90,0x60,0xAC,false, true};
ShopCodeOffsets_s off_Nooklings[5] = {{0x84,0x88,0x5C,0x26C,true, false},{0x10C,0xFC,0xCC,0x26C,false, true},{0x168,0x16C,0x140,0x26C,true, false},{0x1F0,0x1E0,0x1B0,0x26C,false, true},{0x24C,0x250,0x224,0x26C,true, false}};
ShopCodeOffsets_s off_Garden = {0x88,0x8C,0x60,0xB4,true, false};
ShopCodeOffsets_s off_Ables = {0x5C,0x60,0x34,0x7C,true, false};
ShopCodeOffsets_s off_Sham = {0x68,0x6C,0x34,0x48,true, false};
ShopCodeOffsets_s off_Kicks = {0x5C,0x60,0x34,0x9C,true, false};
ShopCodeOffsets_s off_Nooks = {0xA8,0xAC,0x80,0xD0,true, false};
ShopCodeOffsets_s off_Katrina = {0x8C,0x90,0x64,0xAC,true, false};
ShopCodeOffsets_s off_Redd = {0xD0,0xC0,0x90,0xDC,false, true};

static const QVector<PatchValues> NOPVec({{NOP, 0}});
static const QVector<PatchValues> FFVec({{0xFFFFFFFF, 0}});

//Unused Japanese Debug Text in .rodata
static const QVector<quint8> UnusedRoData_Pattern = {
0xE9, 0x80, 0x9A, 0xE4, 0xBF, 0xA1, 0xE3, 0x82,
0xBB, 0xE3, 0x83, 0xBC, 0xE3, 0x83, 0x96
};

//Unused functions in .text
static const QVector<quint8> UnusedCode_Pattern = {
0x14, 0x20, 0xA0, 0xE3, 0xD0, 0x00, 0x80, 0xE5,
0xD8, 0x20, 0x80, 0xE5, 0x28, 0xC0, 0x8F, 0xE2,
0xD4, 0x30, 0x80, 0xE5, 0x01, 0x40, 0xA0, 0xE3,
0xBC, 0xC0, 0x80, 0xE5
};

static const QVector<quint8> NoResetti1Pattern = {
0x57, 0x0C, 0x80, 0xE2, 0x0A, 0x10, 0xD0, 0xE5,
0x02, 0x10, 0x81, 0xE3, 0x0A, 0x10, 0xC0, 0xE5
};

static const QVector<quint8> NoResetti2Pattern = {
0xF0, 0x4F, 0x2D, 0xE9, 0xA7, 0xDF, 0x4D, 0xE2,
0x01, 0x40, 0xA0, 0xE1, 0x00, 0xB0, 0xA0, 0xE1
};

static const QVector<quint8> GetBasePlayerAddrPattern = { //ARM, -0x8
0x00, 0x00, 0x50, 0xE3, 0x08, 0x00, 0x9F, 0x05,
0x00, 0x00, 0xA0, 0x13, 0x00, 0x00, 0x90, 0x05
};

static const QVector<quint8> wcstoulAddrPattern = {
0x25, 0x88, 0xA4, 0x1C,
0x00, 0x2D, 0x04, 0xD0
};

static const QVector<quint8> ConfettiPattern = {
0x16, 0x5B, 0x84, 0xE2, 0xAC, 0x50, 0x85, 0xE2,
0x00, 0x00, 0x95, 0xE5, 0x00, 0x00, 0x50, 0xE3
};

static const QVector<quint8> CherryBPattern = {
0x10, 0xD0, 0x4D, 0xE2, 0x02, 0x00, 0xA0, 0xE3,
0x16, 0x6B, 0x84, 0xE2, 0xAC, 0x60, 0x86, 0xE2
};

static const QVector<quint8> CherryBTreePattern = {
0x00, 0x80, 0xA0, 0xE1, 0x20, 0x10, 0xA0, 0xE3,
0x50, 0x00, 0x8D, 0xE2, 0xF8, 0x04, 0xCD, 0xE1
};

static const QVector<quint8> XmasTreePattern = { //Thumb
0xF8, 0xB5, 0x7B, 0x21,
0x14, 0x46, 0x06, 0x9E
};

static const QVector<quint8> Island3NumsPattern = { //ARM
0x56, 0x10, 0xA0, 0xE3, 0x4C, 0x40, 0x86, 0xE2,
0x00, 0x10, 0x80, 0xE5, 0x11, 0x1E, 0xA0, 0xE3
};

static const QVector<quint8> AllToursPattern = {
0xF0, 0x4F, 0x2D, 0xE9, 0x00, 0x50, 0xA0, 0xE1,
0x00, 0xB0, 0xA0, 0xE3, 0x02, 0x8B, 0x2D, 0xED
};

static const QVector<quint8> MusicEchoPattern = {
0x6C, 0x02, 0x95, 0xE5, 0x06, 0x00, 0x50, 0xE1,
0x01, 0x00, 0x76, 0x13
};

static const QVector<quint8> SaveChecksPattern = {
0x08, 0xD0, 0x4D, 0xE2, 0x0B, 0xB0, 0xA0, 0xE3,
0x05, 0x80, 0xA0, 0xE3, 0x0C, 0x60, 0xA0, 0xE3
};

static const QVector<quint8> RegionChecksPattern = {
0x62, 0x5A, 0x84, 0xE2, 0x4E, 0x01, 0xD5, 0xE5,
0x0F, 0x00, 0x00, 0xE2
};

static const QVector<quint8> InstantTextPattern = {
0x08, 0x10, 0x8D, 0xE5, 0x08, 0x10, 0x8D, 0xE2,
0x04, 0x00, 0xA0, 0xE1
};

static const QVector<quint8> FlowersNoWiltPattern = { //Thumb
0x0E, 0x21, 0x06, 0x90, 0x41, 0x56, 0x00, 0x29,
0x07, 0x91
};

static const QVector<quint8> CameraZoomOutPattern = { //Float; +0x14
0x05, 0x1A, 0xC0, 0xED, 0x06, 0x0A, 0x80, 0xED,
0x0A, 0x1A, 0x80, 0xED, 0x0B, 0x3A, 0xC0, 0xED,
0x1E, 0xFF, 0x2F, 0xE1
};

/*
 * Pattern for 'MoveVillagerOutOfTown' function;
 * This is called by only 'CheckVillagersThatNeedToMoveOut' in code.bin which we also patch
 * But just incase a CRO uses this function or something, give it the same patch
 */
static const QVector<quint8> VillagersNeverMovePattern = { //Thumb
0x06, 0x98, 0x05, 0x99, 0x0A, 0x28
};

//Pattern for 'CheckVillagersThatNeedToMoveOut' function
static const QVector<quint8> VillagersNeverMovePattern2 = { //Thumb
0x2F, 0x18, 0x03, 0x20,
0x38, 0x56, 0x00, 0x28
};

/*
 * Custom Function to replace the 'MoveVillagerOutOfTown' function
 * This function removes the boxed status from all villagers
*/
static const QVector<quint8> VillagersNeverMoveFunction = {
0x01, 0xA1, 0x08, 0x47, // BX to our actual function; Needed to go from Thumb -> ARM
0x00, 0x1C, 0x70, 0x47, //Padding (MOV R0, R0 and BX LR in Thumb just incase somehow used)
0x00, 0xC0, 0xE0, 0xE3, 0x30, 0x20, 0x9F, 0xE5,
0x25, 0x3C, 0x80, 0xE2, 0x2C, 0x10, 0x9F, 0xE5,
0x02, 0xC0, 0xC0, 0xE7, 0x10, 0x30, 0x83, 0xE2,
0x66, 0x0B, 0x80, 0xE2, 0x00, 0x20, 0x93, 0xE5,
0x01, 0x20, 0x02, 0xE0, 0x00, 0x20, 0x83, 0xE5,
0x25, 0x3C, 0x83, 0xE2, 0x18, 0x30, 0x83, 0xE2,
0x03, 0x00, 0x50, 0xE1, 0xF8, 0xFF, 0xFF, 0x1A,
0x1E, 0xFF, 0x2F, 0xE1, //End of Function Code
0x63, 0x97, 0x01, 0x00, 0xFC, 0xF9, 0xFF, 0xFF //Const values
};

static const QVector<quint8> NoGrassDecayPattern = { //Thumb
0x40, 0x18, 0x00, 0x90,
0x31, 0x46, 0x96, 0xA8
};

static const QVector<quint8> WalkOverThingsPattern = { //+0x14
0x0C, 0x20, 0x8D, 0xE2, 0x18, 0x10, 0x8D, 0xE2,
0x04, 0x00, 0xA0, 0xE1
};

static const QVector<quint8> QRAlwaysUnlockedPattern = { //-0xC
0x01, 0x50, 0x80, 0xE0, 0x06, 0x00, 0xD5, 0xE5,
0xA0, 0x03, 0xB0, 0xE1, 0x00, 0x40, 0xA0, 0x03
};

static const QVector<quint8> Permit100Pattern1 = { //+0xC
0x1A, 0x10, 0xD0, 0xE5, 0x1B, 0x20, 0xD0, 0xE5,
0xFE, 0x10, 0xC1, 0xE3, 0x14, 0x10, 0x81, 0xE3
};

static const QVector<quint8> Permit100Pattern2 = { //+0x8
0xFE, 0x20, 0xC3, 0xE3, 0x0E, 0x4D, 0x01, 0xE2,
0x81, 0x10, 0x82, 0xE1, 0x07, 0x30, 0xCC, 0xE3
};

static const QVector<quint8> InstantCyrusCustomsPattern = { //+0
0x03, 0x70, 0xA0, 0xE1, 0x27, 0x00, 0x00, 0x2A,
0x04, 0x20, 0x85, 0xE0, 0x01, 0x00, 0xA0, 0xE3
};

//Check if button is newly pressed
static const QVector<quint8> hidKeysDownPattern = { // -0x30 for function start
0x00, 0x00, 0xA0, 0x93, 0x04, 0x00, 0x90, 0xE5,
0x04, 0x00, 0x10, 0xE0, 0x01, 0x00, 0xA0, 0x13
};

//Check if button is being held
static const QVector<quint8> hidKeysHeldPattern = { // -0x38 for function start
0x0C, 0x00, 0x90, 0xE5, 0x01, 0x00, 0x80, 0xE1,
0x04, 0x00, 0x10, 0xE0, 0x01, 0x00, 0xA0, 0x13
};

//Check if button is newly released
static const QVector<quint8> hidKeysUpPattern = { // -0x30 for function start
0x00, 0x00, 0xA0, 0x93, 0x08, 0x00, 0x90, 0xE5,
0x04, 0x00, 0x10, 0xE0, 0x01, 0x00, 0xA0, 0x13
};

static const QVector<quint8> BTN_SaveMenuPattern = { //+4
0x24, 0x00, 0x9F, 0xE5, 0x40, 0x10, 0xA0, 0xE3,
0x00, 0x00, 0x90, 0xE5
};

static const QVector<quint8> BTN_ScreenshotPattern = { //+0x10 for L, +0x24 for R
0x00, 0x04, 0x94, 0xE5, 0x00, 0x00, 0x50, 0xE3,
0x01, 0x00, 0x40, 0xC2, 0x00, 0x00, 0x81, 0xC5
};

/*
RunB: +0x94; RunL: +0xAC; RunR: +0xC0; Look Up +0x154; ItemSwitchR: +0x1D4
ItemSwitchL: +0x208; Unequip: +0x240; Interact: +0x11C; Pickup: +0xE4
*/
static const QVector<quint8> BTN_General = { //0x68D140 in EUR orig 1.5
0x11, 0x00, 0xD5, 0xE5, 0x11, 0x00, 0xC4, 0xE5,
0x12, 0x00, 0xD5, 0xE5, 0x12, 0x00, 0xC4, 0xE5,
0x13, 0x00, 0xD5, 0xE5, 0x13, 0x00, 0xC4, 0xE5,
0x14, 0xD0, 0x8D, 0xE2, 0x00, 0x00, 0xA0, 0xE3
};

static const QVector<quint8> NoScareFishyPattern = { //+0x10
0x90, 0x51, 0x90, 0xE5, 0x01, 0x60, 0xA0, 0xE1,
0x00, 0x40, 0xA0, 0xE1
};

static const QVector<quint8> FishyBiteWhenPattern = { //-0x4
0xFF, 0x50, 0x00, 0xE2, 0xCE, 0x01, 0xD4, 0xE5,
0x00, 0x01, 0x80, 0xE0, 0x00, 0x01, 0x55, 0xE1
};

/*
 * FishyBiteWhenCode not actually a function, but custom code that's injected into a function

MOV     R5, #0x00       ; This is how many bites (max bites) (we manually change the 0 to user selection)
LDRB    R0, [R4,#0x1CE] ; This is where we'll store current amount of bites (i.e. repurpose)
CMP     R5, R0          ; Compare current bites to max
ADDLT   R0, #1          ; Add 1 to cur bites if less than max
STRB    R0, [R4,#0x1CE] ; This new current bites
*/

static const QVector<quint8> FishyBiteWhenCode = {
0x00, 0x50, 0xA0, 0xE3, 0xCE, 0x01, 0xD4, 0xE5,
0x00, 0x00, 0x55, 0xE1, 0x01, 0x00, 0x80, 0xB2,
0xCE, 0x01, 0xC4, 0xE5
};

static const QVector<quint8> CmdKeyboardHookPattern = { //-0x10
0x4C, 0x00, 0x94, 0xE5, 0x4C, 0x50, 0x84, 0xE2,
0x08, 0x10, 0x90, 0xE5, 0x05, 0x00, 0xA0, 0xE1
};

static const QVector<quint8> CmdKeyboard_Function = {
    /* Loader function; Allows us to do just BL in the game's function instead of LDR and BL; saves one instruction place */
    0xFE, 0x5F, 0x2D, 0xE9, //STMFD SP!, {R1-R12,LR}
    0x50, 0x00, 0x94, 0xE5, //LDR R0, [R4,#0x50]: Get wchar_t pointer for keyboard text
    0x04, 0x10, 0x8F, 0xE2, //ADD R1, PC, #4 //+4 instead of 12 due to how PC works teehee (PC = cur instruction + 8)
    0x31, 0xFF, 0x2F, 0xE1, //BLX R1 //Branch to the code below
    0xFE, 0x9F, 0xBD, 0xE8, //LDMFD SP!, {R1-R12,PC}

    /* Actual function */
    0x0d, 0xc0, 0xa0, 0xe1, 0xf0, 0xdf, 0x2d, 0xe9, 0x04, 0xb0, 0x4c, 0xe2, 0x14, 0xd0, 0x4d, 0xe2,
    0x00, 0xc0, 0xa0, 0xe1, 0xb0, 0x40, 0x9f, 0xe5, 0x34, 0xff, 0x2f, 0xe1, 0x00, 0x00, 0x50, 0xe3,
    0x30, 0x00, 0x0b, 0xe5, 0x38, 0x40, 0x0b, 0xe5, 0x02, 0x00, 0x00, 0xca, 0x01, 0x00, 0xa0, 0xe3,
    0x28, 0xd0, 0x4b, 0xe2, 0xf0, 0xaf, 0x9d, 0xe8, 0xb0, 0x00, 0xdc, 0xe1, 0x2f, 0x00, 0x50, 0xe3,
    0xf9, 0xff, 0xff, 0x1a, 0x84, 0x50, 0x9f, 0xe5, 0x84, 0xa0, 0x9f, 0xe5, 0x30, 0x90, 0x85, 0xe2,
    0x09, 0x70, 0xa0, 0xe1, 0x02, 0xc0, 0x8c, 0xe2, 0x3c, 0x80, 0x85, 0xe2, 0x09, 0x00, 0x47, 0xe0,
    0x34, 0x00, 0x0b, 0xe5, 0x04, 0x00, 0x95, 0xe4, 0x01, 0x00, 0x70, 0xe3, 0xee, 0xff, 0xff, 0x0a,
    0xd1, 0x40, 0xd7, 0xe0, 0x04, 0x00, 0x98, 0xe4, 0x38, 0x60, 0x1b, 0xe5, 0x36, 0xff, 0x2f, 0xe1,
    0x00, 0x00, 0x54, 0xe3, 0x00, 0x60, 0xa0, 0xe1, 0xf3, 0xff, 0xff, 0xda, 0x30, 0x00, 0x1b, 0xe5,
    0x00, 0x00, 0x54, 0xe1, 0xf0, 0xff, 0xff, 0xaa, 0x0c, 0x00, 0xa0, 0xe1, 0x06, 0x20, 0xa0, 0xe1,
    0x04, 0x10, 0x18, 0xe5, 0x3a, 0xff, 0x2f, 0xe1, 0x00, 0x40, 0x50, 0xe2, 0xea, 0xff, 0xff, 0x1a,
    0x18, 0x00, 0x9f, 0xe5, 0x34, 0xe0, 0x1b, 0xe5, 0x0e, 0x51, 0x90, 0xe7, 0x86, 0x00, 0x8c, 0xe0,
    0x35, 0xff, 0x2f, 0xe1, 0x04, 0x00, 0xa0, 0xe1, 0xd8, 0xff, 0xff, 0xea,
    0x35, 0x08, 0x10, 0x00, //wcslen
    0x00, 0x00, 0x00, 0x00, //rodata offset here
    0x65, 0x18, 0x10, 0x00 //wcsncmp
};

static const QVector<quint8> CmdKeyboard_FunctionRoData = {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, //func_ptrs
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,

    0x05, 0x07, 0x07, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, //func_cmd_size
    0x6c, 0x00, 0x00, 0x00, 0x76, 0x00, 0x00, 0x00, 0x82, 0x00, 0x00, 0x00, 0x92, 0x00, 0x00, 0x00, //func_cmds
    0xa4, 0x00, 0x00, 0x00, 0xac, 0x00, 0x00, 0x00, 0xb4, 0x00, 0x00, 0x00, 0xbc, 0x00, 0x00, 0x00,
    0xc4, 0x00, 0x00, 0x00, 0xcc, 0x00, 0x00, 0x00, 0xd4, 0x00, 0x00, 0x00, 0xdc, 0x00, 0x00, 0x00,

    //Let's make tons of space for our strings
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,

};

static const QVector<quint8> CmdKeyboard_Text2Item_Function = {
    0x0D, 0xC0, 0xA0, 0xE1, 0x00, 0x10, 0xA0, 0xE3, 0x00, 0xD8, 0x2D, 0xE9, 0x10, 0x20, 0xA0, 0xE3,
    0x58, 0x30, 0x9F, 0xE5, 0x04, 0xB0, 0x4C, 0xE2, 0x33, 0xFF, 0x2F, 0xE1, 0x00, 0x30, 0xA0, 0xE1,
    0x02, 0x29, 0xC0, 0xE3, 0x48, 0x00, 0x9F, 0xE5, 0x30, 0xFF, 0x2F, 0xE1, 0x00, 0x00, 0x52, 0xE3,
    0x00, 0x00, 0x50, 0x13, 0x00, 0xA8, 0x9D, 0x08, 0x38, 0x10, 0x9F, 0xE5, 0x83, 0x38, 0xA0, 0xE1,
    0xA3, 0x38, 0xA0, 0xE1, 0x01, 0x00, 0x53, 0xE1, 0xF9, 0xC0, 0x43, 0xE2, 0x00, 0x30, 0xA0, 0x93,
    0x01, 0x30, 0xA0, 0x83, 0x20, 0x10, 0x9F, 0xE5, 0x01, 0x00, 0x5C, 0xE1, 0x01, 0x30, 0x83, 0x93,
    0x00, 0x00, 0x53, 0xE3, 0x6B, 0x0C, 0x80, 0x02, 0xD0, 0x20, 0x80, 0x05, 0x00, 0xA8, 0x9D, 0xE8,
    0xC9, 0x13, 0x10, 0x00, //wcstoul
    0xC8, 0xBA, 0x2F, 0x00, //Get_Base_Player_Address
    0xFE, 0x7F, 0x00, 0x00, 0x07, 0x1F, 0x00, 0x00
};

static const QVector<quint8> CmdKeyboard_ChangeTime_Function = {
    0x0D, 0xC0, 0xA0, 0xE1, 0x00, 0xD8, 0x2D, 0xE9, 0x04, 0xB0, 0x4C, 0xE2, 0x00, 0xA8, 0x9D, 0xE8
};

static const QVector<quint8> CmdKeyboard_DupeAll_Function = {
    0x0D, 0xC0, 0xA0, 0xE1, 0x34, 0x00, 0x9F, 0xE5, 0x00, 0xD8, 0x2D, 0xE9, 0x04, 0xB0, 0x4C, 0xE2,
    0x30, 0xFF, 0x2F, 0xE1, 0x00, 0x00, 0x50, 0xE3, 0x00, 0xA8, 0x9D, 0x08, 0x6B, 0x2C, 0x80, 0xE2,
    0xD0, 0x30, 0x82, 0xE2, 0x1B, 0x0B, 0x80, 0xE2, 0xD0, 0x20, 0x92, 0xE5, 0x10, 0x00, 0x80, 0xE2,
    0x04, 0x20, 0x83, 0xE4, 0x03, 0x00, 0x50, 0xE1, 0xFC, 0xFF, 0xFF, 0x1A, 0x00, 0xA8, 0x9D, 0xE8,
    0xC8, 0xBA, 0x2F, 0x00 //Get_Base_Player_Address
};

static const QVector<quint8> CmdKeyboard_ClearInv_Function = {
    0x0d, 0xc0, 0xa0, 0xe1, 0x34, 0x00, 0x9f, 0xe5, 0x00, 0xd8, 0x2d, 0xe9, 0x04, 0xb0, 0x4c, 0xe2,
    0x30, 0xff, 0x2f, 0xe1, 0x00, 0x00, 0x50, 0xe3, 0x00, 0xa8, 0x9d, 0x08, 0x6b, 0x3c, 0x80, 0xe2,
    0x1c, 0x20, 0x9f, 0xe5, 0x1b, 0x0b, 0x80, 0xe2, 0xd0, 0x30, 0x83, 0xe2, 0x10, 0x00, 0x80, 0xe2,
    0x04, 0x20, 0x83, 0xe4, 0x03, 0x00, 0x50, 0xe1, 0xfc, 0xff, 0xff, 0x1a, 0x00, 0xa8, 0x9d, 0xe8,
    0xc8, 0xba, 0x2f, 0x00, 0xfe, 0x7f, 0x00, 0x00
};

/* General Utils */
Patch UnusedCode;
Patch UnusedRoData;
Patch GetBasePlayerAddr; //Pattern: 00 00 50 E3 08 00 9F 05 00 00 A0 13 00 00 90 05 - 0x8
Patch wcstoulAddr; //Thumb; Pattern: 25 88 A4 1C 00 2D 04 D0 - 0x14

/* Button Remapper Utils */
Patch hidKeysDown;
Patch hidKeysHeld;
Patch hidKeysUp;

/* Command Keyboard Utils */
Patch CmdKeyboardFunction;
Patch CmdKeyboardRoData;

Patch CmdKeyboard_Text2Item;
Patch CmdKeyboard_ChangeTime;
Patch CmdKeyboard_DupeAll;
Patch CmdKeyboard_ClearInv;

/* Exefs->General */
Patch Confetti; //Pattern: 16 5B 84 E2 AC 50 85 E2  00 00 95 E5 00 00 50 E3 + 0x8
Patch CherryBlossom; //Pattern: 10 D0 4D E2 02 00 A0 E3  16 6B 84 E2 AC 60 86 E2 + 0x8
Patch Weather; //Pattern: 1st find of [06 10 41 E2 00 00 51 E3  18 10 81 B2 34 00 50 E3] + 0x88
Patch Season; //Pattern: 1st find of [00 00 A0 E3 0C 34 83 E1 00 10 A0 E1 73 30 FF E6] + 0x18
Patch CherryBTrees; //Pattern: 00 80 A0 E1  20 10 A0 E3 50 00 8D E2 + 0x0
Patch AlwaysXmasTrees; //Thumb; Pattern: F8 B5 7B 21 14 46 06 9E + 0xA
Patch Island3Nums; //Pattern: 56 10 A0 E3 4C 40 86 E2 00 10 80 E5 11 1E A0 E3 - 0x10
Patch PickAllTours; //Pattern: F0 4F 2D E9 00 50 A0 E1 00 B0 A0 E3  02 8B 2D ED + 0x8
Patch MusicHasEcho; //Pattern: 6C 02 95 E5 06 00 50 E1 01 00 76 13 + 0xC
Patch VillagersNeverMove; //Thumb; Pattern: 06 98 05 99 0A 28
Patch VillagersNeverMove2; //Thumb; Pattern: 2F 18 03 20 38 56 00 28
Patch NoGrassDecay; //Thumb; Pattern: 40 18 00 90 31 46 96 A8
Patch FlowersNoWilt; //Thumb; Pattern: 0E 21 06 90 41 56 00 29 07 91 - 0x8
Patch OnlyBlackRoseWilts; //Thumb; Pattern: 0E 21 06 90 41 56 00 29 07 91 - 0x10; Specifically for if black roses should still wilt
Patch FishyBiteWhen; // Pattern: FF 50 00 E2 CE 01 D4 E5 00 01 80 E0 00 01 55 E1 - 0x4
Patch NoScareFishy; //Pattern: 90 51 90 E5 01 60 A0 E1 00 40 A0 E1 + 0x10

/* Exefs->Player */
Patch NoResetti1; //Don't set 'ResetPending' flag; Pattern: 57 0C 80 E2 0A 10 D0 E5 02 10 81 E3 0A 10 C0 E5 + 0x8
//NoResetti2: Resetti Never Appears (needed as NoResetti1 doesn't cover all cases);
Patch NoResetti2; //Pattern: F0 4F 2D E9 A7 DF 4D E2 01 40 A0 E1 00 B0 A0 E1 + 0x8
Patch PlayerSpeed;
Patch EditPattern;
Patch FlowersNoTrample;
Patch NoMosquito; //Pattern: 10 40 BD E8 03 20 A0 E1 A6 10 A0 E3
Patch WalkOverThings; //Pattern: 0C 20 8D E2 18 10 8D E2 04 00 A0 E1
Patch QRMachineAlwaysUnlocked; //Pattern: 01 50 80 E0 06 00 D5 E5 A0 03 B0 E1 00 40 A0 03 -0xC
Patch PermitAlways100_1; //New Player starts w/ 100%; Pattern: 1A 10 D0 E5 1B 20 D0 E5 FE 10 C1 E3 14 10 81 E3 + 0xC
Patch PermitAlways100_2; //AddtoPermit func sets 100%; Pattern: FE 20 C3 E3 0E 4D 01 E2 81 10 82 E1 07 30 CC E3 + 0x8

/* Exefs->Utilities */
Patch RegionCheck; //Pattern: 62 5A 84 E2 4E 01 D5 E5 0F 00 00 E2 - 0xC
Patch ChecksumCheck; //Pattern: 08 D0 4D E2 0B B0 A0 E3 05 80 A0 E3 0C 60 A0 E3 + 0xB4
Patch SecureValueCheck; //Pattern: 08 D0 4D E2 0B B0 A0 E3 05 80 A0 E3 0C 60 A0 E3 + 0xE0
Patch InstantText; //Pattern: 08 10 8D E5 08 10 8D E2 04 00 A0 E1 + 0x48
Patch CameraZoomOut; //Float; Pattern: 05 1A C0 ED 06 0A 80 ED  0A 1A 80 ED 0B 3A C0 ED 1E FF 2F E1 + 0x14
Patch CmdKeyboardHook;

/* Exefs->Shops */
Patch NativeFruitPrice;
Patch ReeseBuy;
Patch NooklingsBuy;
Patch LeilaBuy;
Patch InstantCyrusCustoms; //Pattern: 03 70 A0 E1 27 00 00 2A  04 20 85 E0 01 00 A0 E3 + 0

/* Shop Times: Addresses are start of function */
Patch Retail;
Patch Nooklings; //Pattern: 00 10 A0 E1  14 00 D4 E5 04 40 91 E5
Patch Garden;
Patch Ables;
Patch Sham;
Patch Kicks;
Patch Nooks;
Patch Katrina;
Patch Redd;
Patch NightOwl;

/* Button Remapper */
Patch Button_RunB;
Patch Button_RunL;
Patch Button_RunR;
Patch Button_Unequip;
Patch Button_ItemSwitchDL;
Patch Button_ItemSwitchDR;
Patch Button_LookUp;
Patch Button_Pickup;
Patch Button_Interact;
Patch Button_ScreenshotL;
Patch Button_ScreenshotR;
Patch Button_SaveMenu;

Patch::Patch() { }

Patch::Patch(quint32 NL_JPN, quint32 NL_USA, quint32 NL_EUR, quint32 NL_KOR,
      quint32 WA_JPN, quint32 WA_USA, quint32 WA_EUR, quint32 WA_KOR,
      QVector<PatchValues> Values)
{
    quint32 Offsets[8] = {NL_JPN, NL_USA, NL_EUR, NL_KOR, WA_JPN, WA_USA, WA_EUR, WA_KOR};
    Game* game = MainWindow::GetGameInstance();
    if (game == nullptr || game->m_GameType > 8)
        m_Offset = 0;

    else {
        m_Offset = Offsets[game->m_GameType];
        m_Values = Values;
    }
}

Patch::Patch(const QVector<quint8> Pattern, QVector<PatchValues> Values, quint32 OffsetFromPattern) {
    Game* game = MainWindow::GetGameInstance();
    game->codebin->m_file->seek(0);
    QByteArray data = game->codebin->m_file->readAll();

    //Nanquitas search function is 2x faster than std::search in this instance
    quint32 offset = Search<quint8>(data.constData(), data.size(), Pattern);
    if (offset != 0) {
        m_Offset = 0x100000+offset+OffsetFromPattern;
        m_Values = Values;
    }

    else m_Offset = 0;
    qDebug() << "Pattern found offset: 0x" + QString::number(m_Offset, 16);
}

Patch::Patch(const QVector<quint8> Pattern, QVector<quint8> Values, quint32 OffsetFromPattern) {
    Game* game = MainWindow::GetGameInstance();
    game->codebin->m_file->seek(0);
    QByteArray data = game->codebin->m_file->readAll();

    //Nanquitas search function is 2x faster than std::search in this instance
    quint32 offset = Search<quint8>(data.constData(), data.size(), Pattern);
    if (offset != 0) {
        m_Offset = 0x100000+offset+OffsetFromPattern;
        m_Valuesu8 = Values;
    }

    else m_Offset = 0;
    qDebug() << "Pattern found offset: 0x" + QString::number(m_Offset, 16);
}

//Returns false if no patches fail; true if one or more patches fail
bool Patch::Apply(File* code, quint8 OR) {
    bool res = true;
    if (m_Offset == 0) return true;

    if (!m_Values.empty()) {
        res = false;
        for (auto patch : this->m_Values) {
            res |= !Game::PatchCode(code, m_Offset + patch.OffsetFromBaseAddr, patch.Value|OR);
        }
    }

    else if (!m_Valuesu8.empty()) {
        res = false;
        QByteArray array(reinterpret_cast<const char*>(this->m_Valuesu8.constData()), this->m_Valuesu8.size());
        res |= !code->Write(m_Offset - 0x100000, array);
    }

    return res;
}

QVector<PatchValues> Patch::MakeARMPatchValuesFromFunctionBytes(QVector<quint8> function) {
    static const quint8 codesize = 4;
    QVector<PatchValues> vec;

    quint32 size = static_cast<quint32>(function.size());
    if ((size % codesize) != 0) { //Check if function bytes has fully completed instructions
        qDebug() << "Function bytes are missing, not full function!";
        return vec;
    }

    quint32 i = 0;
    while (i < size) {
        quint32 instruction = 0;
        for (quint8 j = 0; j < codesize; j++) {
            quint8 byte = function[i+j];
            instruction |= byte<<(8*j);
        }
        vec.push_back({instruction, i});
        i+=4;
    }

    return vec;
}

void Patch::Init(void) {

    /* General Utils */
    UnusedCode =       Patch(UnusedCode_Pattern, QVector<PatchValues>({{0x00000000,0}}), static_cast<quint32>(-8));
    UnusedRoData =     Patch(UnusedRoData_Pattern, QVector<PatchValues>({{0x00000000,0}}), 0);
    while (UnusedRoData.m_Offset % 4) UnusedRoData.m_Offset++; //Align to 4 bytes
    while (UnusedCode.m_Offset % 4) UnusedCode.m_Offset++; //Align to 4 bytes

    GetBasePlayerAddr= Patch(GetBasePlayerAddrPattern, QVector<PatchValues>({{0,0}}), static_cast<quint32>(-8));
    wcstoulAddr      = Patch(wcstoulAddrPattern, QVector<PatchValues>({{0,0}}), static_cast<quint32>(-0x14));
    wcstoulAddr.m_Offset+=1; //Show it's Thumb offset when wanting to jump to it
    qDebug() << "End General Utils";

    /* Command Keyboard Utils */
    CmdKeyboardFunction=Patch(UnusedCode_Pattern, MakeARMPatchValuesFromFunctionBytes(CmdKeyboard_Function), 0);
    CmdKeyboardRoData = Patch(UnusedRoData_Pattern, CmdKeyboard_FunctionRoData, 0);
    CmdKeyboard_Text2Item = Patch(UnusedCode_Pattern, MakeARMPatchValuesFromFunctionBytes(CmdKeyboard_Text2Item_Function),0);
    CmdKeyboard_ChangeTime = Patch(UnusedCode_Pattern, MakeARMPatchValuesFromFunctionBytes(CmdKeyboard_ChangeTime_Function),0);
    CmdKeyboard_DupeAll = Patch(UnusedCode_Pattern, MakeARMPatchValuesFromFunctionBytes(CmdKeyboard_DupeAll_Function),0);
    CmdKeyboard_ClearInv = Patch(UnusedCode_Pattern, MakeARMPatchValuesFromFunctionBytes(CmdKeyboard_ClearInv_Function),0);
    qDebug() << "End Cmd Keyboard Utils";

    /* Button Remapper Utils */
    hidKeysDown =       Patch(hidKeysDownPattern, QVector<PatchValues>({{0xE92D4010,0}}), static_cast<quint32>(-0x30));
    hidKeysHeld =       Patch(hidKeysHeldPattern, QVector<PatchValues>({{0xE92D4010,0}}), static_cast<quint32>(-0x38));
    hidKeysUp =         Patch(hidKeysUpPattern, QVector<PatchValues>({{0xE92D4010,0}}), static_cast<quint32>(-0x30));
    qDebug() << "End Button Remap Utils";

    /* Exefs->General */
    Confetti =          Patch(ConfettiPattern, QVector<PatchValues>({{0xE3A00001, 0}, {0xE3A00078, 0x30}}), 8);
    CherryBlossom =     Patch(CherryBPattern, QVector<PatchValues>({{0xE3A00001, 0}, {0xE3A00001, 0x28}, {0xE3A00004, 0x50}, {0xE3A01001, 0x60}}), 0x10);
    Weather =           Patch(0x62E728, 0x62FC30, 0x62EC68, KOR, 0x62E728, 0x62F158, 0x62EC68, WAKOR, QVector<PatchValues>({{0xE3A00000,0}})); //Pattern: 1st find of [06 10 41 E2 00 00 51 E3  18 10 81 B2 34 00 50 E3] + 0x88
    Season =            Patch(0x56B090, 0x56C758, 0x56B7A0, KOR, 0x56B090, 0x56BC70, 0x56B7A0, WAKOR, QVector<PatchValues>({{0xE3A01000,0}})); //Pattern: 1st find of [00 00 A0 E3 0C 34 83 E1 00 10 A0 E1 73 30 FF E6] + 0x18
    CherryBTrees =      Patch(CherryBTreePattern, QVector<PatchValues>({{0xE3A08005,0}, {0xE28F2074, 0x150}}), 0);
    AlwaysXmasTrees =   Patch(XmasTreePattern, QVector<PatchValues>({{0x1C002000, 0}}), 0xA);
    Island3Nums =       Patch(Island3NumsPattern, NOPVec, static_cast<quint32>(-0x10));
    PickAllTours =      Patch(AllToursPattern, QVector<PatchValues>({{0xE3E0B000, 0}}), 8);
    MusicHasEcho =      Patch(MusicEchoPattern, QVector<PatchValues>({{0xE1A00000,0}, {0xE3A01003, 4}}), 0xC);
    VillagersNeverMove= Patch(VillagersNeverMovePattern, MakeARMPatchValuesFromFunctionBytes(VillagersNeverMoveFunction), static_cast<quint32>(-4));
    VillagersNeverMove2=Patch(VillagersNeverMovePattern2, MakeARMPatchValuesFromFunctionBytes(VillagersNeverMoveFunction), static_cast<quint32>(-0xA));
    NoGrassDecay =      Patch(NoGrassDecayPattern, QVector<PatchValues>({{0x980022FF, 0}, {0x980022FF, 0x12}}), 0xC);
    FlowersNoWilt =     Patch(FlowersNoWiltPattern, QVector<PatchValues>({{0xE04A9004, 0}}), static_cast<quint32>(-0x8));
    OnlyBlackRoseWilts= Patch(FlowersNoWiltPattern, QVector<PatchValues>({{0x900428B9, 0}, {0xD14A9004, 2}}), static_cast<quint32>(-0xA)); //If people still want Black Roses to wilt
    FishyBiteWhen =     Patch(FishyBiteWhenPattern, MakeARMPatchValuesFromFunctionBytes(FishyBiteWhenCode), static_cast<quint32>(-4));
    NoScareFishy =      Patch(NoScareFishyPattern, QVector<PatchValues>({{0xE3500001, 0}}), 0x10);
    qDebug() << "End Exefs->General";

    /* Exefs->Player */
    NoResetti1 =        Patch(NoResetti1Pattern, QVector<PatchValues>({{0xE3C11002, 0}}), 8);
    NoResetti2 =        Patch(NoResetti2Pattern, QVector<PatchValues>({{0xE3A04000, 0}}), 8);
    PlayerSpeed =       Patch(0x651708, 0x652C10, 0x651C48, KOR, 0x651708, 0x652138, 0x651C48, WAKOR, QVector<PatchValues>({{0x3F800000, 0}}));
    EditPattern =       Patch(0x2FEC78, 0x2FEC44, 0x2FECCC, KOR, 0x2FEC78, 0x2FE9C0, 0x2FECCC, WAKOR, QVector<PatchValues>({{0xE3A00001, 0}}));
    FlowersNoTrample =  Patch(0x596890, 0x597F58, 0x596FA0, KOR, 0x596890, 0x597470, 0x596FA0, WAKOR, QVector<PatchValues>({{0xE3A0001D, 0}}));
    NoMosquito =        Patch(0x5C245C, 0x5C3B24, 0x5C2B6C, KOR, 0x5C245C, 0x5C3054, 0x5C2B6C, WAKOR, QVector<PatchValues>({{0xE3A01006,0}})); //Pattern: 10 40 BD E8 03 20 A0 E1 A6 10 A0 E3
    WalkOverThings =    Patch(WalkOverThingsPattern, QVector<PatchValues>({{0xEA000094, 0}, {0xE1A00000, 0x2A8}, {0xE1A00000, 0x2C0}, {0xEA000026, 0x2C4}, {0xEA000065, 0x2F0}}), 0x14);
    QRMachineAlwaysUnlocked = Patch(QRAlwaysUnlockedPattern, QVector<PatchValues>({{0xE3A00000, 0x10}, {0xE3A04000, 0x18}, {0xE1A00000, 0x20},
                                                                                   {0xE5D06000, 0x40}, {0xE3C66078, 0x44}, {0xE3866050, 0x48},
                                                                                   {0xE5C06000, 0x4C}, {0xE1A00000, 0x5C}}), static_cast<quint32>(-0xC));
    PermitAlways100_1 = Patch(Permit100Pattern1,  QVector<PatchValues>({{0xE38110C8, 0}}), 0xC);
    PermitAlways100_2 = Patch(Permit100Pattern2, QVector<PatchValues>({{0xE38210C8, 0}}), 0x8);
    qDebug() << "End Exefs->Player";

    /* Exefs->Utilities */
    RegionCheck =       Patch(RegionChecksPattern, QVector<PatchValues>({{0xE1A00005,0}}), static_cast<quint32>(-0xC));
    ChecksumCheck =     Patch(SaveChecksPattern, QVector<PatchValues>({{0xE3A00001,0}, {0xE3A00001,0x1C}}), 0xB4);
    SecureValueCheck =  Patch(SaveChecksPattern, QVector<PatchValues>({{0xE3A00001, 0}}), 0xE0);
    InstantText =       Patch(InstantTextPattern, QVector<PatchValues>({{0xE3A00001, 0}}), 0x48);
    CameraZoomOut =     Patch(CameraZoomOutPattern, QVector<PatchValues>({{0x3F800000, 0}}), 0x14);
    CmdKeyboardHook =   Patch(CmdKeyboardHookPattern, QVector<PatchValues>({{0xE3A00000, 0}, {0xE3500000, 4}, {0xE59F6064, 8}, {0x15960000, 0xC}}), static_cast<quint32>(-0x10));
    qDebug() << "End Exefs->Utilities";

    /* Exefs->Shops */
    NativeFruitPrice =  Patch(0x30576C, 0x3056AC, 0x3056B8, KOR, 0x30576C, 0x305714, 0x3056B8, WAKOR, NOPVec);
    ReeseBuy =          Patch(0x768EE0, 0x76A740, 0x769748, KOR, 0x768EB8, 0x769724, 0x769720, WAKOR, NOPVec);
    NooklingsBuy =      Patch(0x769148, 0x76A9A8, 0x7699B0, KOR, 0x769120, 0x76998C, 0x769988, WAKOR, QVector<PatchValues>({{0xE1A041A0, 0}, {NOP, 8}}));
    LeilaBuy =          Patch(0x768884, 0x76A0E4, 0x7690EC, KOR, 0x76885C, 0x7690C8, 0x7690C4, WAKOR, QVector<PatchValues>({{0xE1A00004,0}}));
    InstantCyrusCustoms=Patch(InstantCyrusCustomsPattern, QVector<PatchValues>({{0xE1A07002, 0}, {0xE3A00000, 0xC}}),0);
    qDebug() << "End Exefs->Shops";


    /* Shop Times: Addresses are start of function */
    Retail =            Patch(0x309310, 0x30929C, 0x309298, KOR, 0x309310, 0x309384, 0x309298, WAKOR, FFVec);
    Nooklings =         Patch(0x710054, 0x7118A0, 0x7108A8, KOR, 0x71002C, 0x710D50, 0x710880, WAKOR, FFVec); //Pattern: 00 10 A0 E1  14 00 D4 E5 04 40 91 E5
    Garden =            Patch(0x7102D0, 0x711B1C, 0x710B24, KOR, 0x7102A8, 0x710FCC, 0x710AFC, WAKOR, FFVec);
    Ables =             Patch(0x7125E8, 0x713E34, 0x712E3C, KOR, 0x7125C0, 0x7132E4, 0x712E14, WAKOR, FFVec);
    Sham =              Patch(0x71BB98, 0x71D3E4, 0x71C3EC, KOR, 0x71BB70, 0x71C72C, 0x71C3C4, WAKOR, FFVec);
    Kicks =             Patch(0x70FF64, 0x7117B0, 0x7107B8, KOR, 0x70FF3C, 0x710C60, 0x710790, WAKOR, FFVec);
    Nooks =             Patch(0x71DD38, 0x71F584, 0x71E58C, KOR, 0x71DD10, 0x71E8CC, 0x71E564, WAKOR, FFVec);
    Katrina =           Patch(0x7167A0, 0x717FEC, 0x716FF4, KOR, 0x716778, 0x71749C, 0x716FCC, WAKOR, FFVec);
    Redd =              Patch(0x716B1C, 0x718368, 0x717370, KOR, 0x716AF4, 0x717818, 0x717348, WAKOR, FFVec);
    NightOwl =          Patch(0x7585C4, 0x759E24, 0x758E2C, KOR, 0x75859C, 0x758E08, 0x758E04, WAKOR, FFVec);
    qDebug() << "End Shop Times";

    /* Button Remap */
    Button_RunB =       Patch(BTN_General, QVector<PatchValues>({{0xE3A00002, 0}}), 0x94);
    Button_RunL =       Patch(BTN_General, QVector<PatchValues>({{0xE3A00A02, 0}}), 0xAC);
    Button_RunR =       Patch(BTN_General, QVector<PatchValues>({{0xE3A00901, 0}}), 0xC0);
    Button_Unequip =    Patch(BTN_General, QVector<PatchValues>({{0xE3A00802, 0}}), 0x240);
    Button_ItemSwitchDL=Patch(BTN_General, QVector<PatchValues>({{0xE3A00701, 0}}), 0x208);
    Button_ItemSwitchDR=Patch(BTN_General, QVector<PatchValues>({{0xE3A00702, 0}}), 0x1D4);
    Button_LookUp =     Patch(BTN_General, QVector<PatchValues>({{0xE3A00801, 0}}), 0x154);
    Button_Pickup =     Patch(BTN_General, QVector<PatchValues>({{0xE3A00010, 0}}), 0xE4);
    Button_Interact =   Patch(BTN_General, QVector<PatchValues>({{0xE3A00001, 0}}), 0x11C);

    Button_ScreenshotL= Patch(BTN_ScreenshotPattern, QVector<PatchValues>({{0xE3A00A02, 0}}), 0x10);
    Button_ScreenshotR= Patch(BTN_ScreenshotPattern, QVector<PatchValues>({{0xE3A00901, 0}}), 0x24);
    Button_SaveMenu =   Patch(BTN_SaveMenuPattern, QVector<PatchValues>({{0xE1A00000, 0}, {0xE3A00B02, 4}, {0xEB0E02A8, 8}}), 4);
    qDebug() << "End Button Remap";
}
